@@ -1,24 +1,29 @@
 lrEM <- function(X,label=NULL,dl=NULL,rob=FALSE,ini.cov=c("complete.obs","multRepl"),delta=0.65,tolerance=0.0001,
-         max.iter=50,rlm.maxit=150,suppress.print=FALSE){
+         max.iter=50,rlm.maxit=150,imp.missing=FALSE,suppress.print=FALSE){
   
-  if (is.character(dl)) stop("dl must be a numeric vector or matrix")
-  if (is.vector(dl)) dl <- matrix(dl,nrow=1)
+  if (imp.missing==FALSE){
+    if (is.character(dl)) stop("dl must be a numeric vector or matrix")
+    if (is.vector(dl)) dl <- matrix(dl,nrow=1)
+  }
   
   if ((is.vector(X)) | (nrow(X)==1)) stop("X must be a data matrix")
   if (is.null(label)) stop("A value for label must be given")
   if (!is.na(label)){
     if (!any(X==label,na.rm=T)) stop(paste("Label",label,"was not found in the data set"))
-    if (label!=0 & any(X==0,na.rm=T)) stop("Zero values not labelled as censored values were found in the data set")
-    if (any(is.na(X))) stop(paste("NA values not labelled as censored values were found in the data set"))
+    if (label!=0 & any(X==0,na.rm=T)) stop("Zero values not labelled as censored or missing values were found in the data set")
+    if (any(is.na(X))) stop(paste("NA values not labelled as censored or missing values were found in the data set"))
   }
   if (is.na(label)){
-    if (any(X==0,na.rm=T)) stop("Zero values not labelled as censored values were found in the data set")
+    if (any(X==0,na.rm=T)) stop("Zero values not labelled as censored or missing values were found in the data set")
     if (!any(is.na(X),na.rm=T)) stop(paste("Label",label,"was not found in the data set"))
   }
-  if (ncol(dl)!=ncol(X)) stop("The number of columns in X and dl do not agree")
-  if ((nrow(dl)>1) & (nrow(dl)!=nrow(X))) stop("The number of rows in X and dl do not agree")
+  if (imp.missing==FALSE){
+    if (ncol(dl)!=ncol(X)) stop("The number of columns in X and dl do not agree")
+    if ((nrow(dl)>1) & (nrow(dl)!=nrow(X))) stop("The number of rows in X and dl do not agree")
+  }
   
   ini.cov <- match.arg(ini.cov)
+  if (imp.missing==TRUE) {ini.cov <- "complete.obs"}
   
   lm.sweep <- function(M,C,varobs){
     
@@ -117,7 +122,6 @@ lrEM <- function(X,label=NULL,dl=NULL,rob=FALSE,ini.cov=c("complete.obs","multRe
     return(x)
   }
   
-  
   ## Preliminaries ----  
   
   X <- as.data.frame(X)
@@ -127,7 +131,7 @@ lrEM <- function(X,label=NULL,dl=NULL,rob=FALSE,ini.cov=c("complete.obs","multRe
   X <- as.data.frame(apply(X,2,as.numeric))
   c <- apply(X,1,sum,na.rm=TRUE)
   
-  if (nrow(dl)==1) dl <- matrix(rep(1,nn),ncol=1)%*%dl
+  if (imp.missing==FALSE) {if (nrow(dl)==1) dl <- matrix(rep(1,nn),ncol=1)%*%dl}
   
   # Check for closure
   closed <- 0
@@ -144,8 +148,10 @@ lrEM <- function(X,label=NULL,dl=NULL,rob=FALSE,ini.cov=c("complete.obs","multRe
     pos <- which(!is.na(colSums(X)))[1]
     if (is.na(pos)) stop("lrEM based on alr requires at least one complete column")
     
+    if (imp.missing==FALSE){
     cpoints <- log(dl)-log(X[,pos])-.Machine$double.eps
     cpoints <- cpoints[,-pos]
+    }
     
     X_alr <- log(X)-log(X[,pos]); X_alr <- as.matrix(X_alr[,-pos])
     tol <- tolerance
@@ -181,8 +187,13 @@ lrEM <- function(X,label=NULL,dl=NULL,rob=FALSE,ini.cov=c("complete.obs","multRe
         varmiss <- which(is.na(X_alr[i[1],]))
         if (length(varobs) == 0){
           alt.in <- TRUE
-          temp <- multRepl(X[i,,drop=FALSE],label=NA,dl=dl[i,,drop=FALSE],delta=delta)
-          Y[i,] <- t(apply(temp,1,function(x) log(x)-log(x[pos])))[,-pos]
+          if (imp.missing==FALSE){
+            temp <- multRepl(X[i,,drop=FALSE],label=NA,dl=dl[i,,drop=FALSE],delta=delta)
+            Y[i,] <- t(apply(temp,1,function(x) log(x)-log(x[pos])))[,-pos]
+          }
+          if (imp.missing==TRUE){
+            stop("Please remove samples with only one observed component (check out using zPatterns).")
+          }
           if (niters == 1){
             alt.pat <- c(alt.pat,npat)
             alt.mr <- list(alt.mr,i)
@@ -194,14 +205,17 @@ lrEM <- function(X,label=NULL,dl=NULL,rob=FALSE,ini.cov=c("complete.obs","multRe
         CR <- lm.sweep(M,C,varobs)[[2]]
         Y[i,varmiss] <- matrix(1,nrow=length(i))%*%B[1,] + X_alr[i, varobs, drop=FALSE]%*%B[2:(length(varobs)+1),]
         sigmas[varmiss] <- sqrt(diag(as.matrix(CR)))
-        for (j in 1:length(varmiss)){                                
-          sigma <- sigmas[varmiss[j]]
-          fdN01 <- dnorm((cpoints[i,varmiss[j]]-Y[i,varmiss[j]])/sigma)
-          fdistN01 <- pnorm((cpoints[i,varmiss[j]]-Y[i,varmiss[j]])/sigma)
-          Y[i,varmiss[j]] <- Y[i,varmiss[j]]-sigma*(fdN01/fdistN01)
+        if (imp.missing==FALSE){
+          for (j in 1:length(varmiss)){
+            sigma <- sigmas[varmiss[j]]
+            fdN01 <- dnorm((cpoints[i,varmiss[j]]-Y[i,varmiss[j]])/sigma)
+            fdistN01 <- pnorm((cpoints[i,varmiss[j]]-Y[i,varmiss[j]])/sigma)
+            Y[i,varmiss[j]] <- Y[i,varmiss[j]]-sigma*(fdN01/fdistN01)
+          }
         }
         v[varmiss,varmiss] <- v[varmiss,varmiss] + CR*length(i)
       }
+      
       M <- matrix(colMeans(Y),ncol=1)
       dif <- Y - matrix(1,nrow=nn)%*%t(M)              
       PC <- t(dif)%*%dif                       
@@ -249,7 +263,12 @@ lrEM <- function(X,label=NULL,dl=NULL,rob=FALSE,ini.cov=c("complete.obs","multRe
       for (npat in 2:length(levels(misspat))){                    
         if ((length(obs[[npat]]) == 1) & (!any(npat==alt.pat))){
           alt.in <- TRUE
-          X[misspat==npat,] <- multRepl(X.old[misspat==npat,,drop=FALSE],label=NA,dl=dl[misspat==npat,,drop=FALSE],delta=delta)    
+          if (imp.missing==FALSE){
+            X[misspat==npat,] <- multRepl(X.old[misspat==npat,,drop=FALSE],label=NA,dl=dl[misspat==npat,,drop=FALSE],delta=delta)    
+          }
+          if (imp.missing==TRUE){
+            stop("Please remove samples with only one observed component (check out using zPatterns).")
+          }
           alt.pat <- c(alt.pat,npat)
           alt.mr <- list(alt.mr,which(misspat==npat))
         }
@@ -258,7 +277,9 @@ lrEM <- function(X,label=NULL,dl=NULL,rob=FALSE,ini.cov=c("complete.obs","multRe
           for (m in 1:length(miss[[npat]])){
             p <- miss[[npat]][m]
             target <- X.old[,p]
-            phi <- t(apply(cbind(dl=dl[misspat==npat,p],feeder[misspat==npat,]),1,ilr))
+            if (imp.missing==FALSE){
+              phi <- t(apply(cbind(dl=dl[misspat==npat,p],feeder[misspat==npat,]),1,ilr))
+            }
             regbasis <- as.data.frame(t(apply(cbind(target,feeder),1,ilr)))
             
             if (niters == 1){
@@ -281,7 +302,9 @@ lrEM <- function(X,label=NULL,dl=NULL,rob=FALSE,ini.cov=c("complete.obs","multRe
             B <- matrix(robreg$coefficients,ncol=1)
             sigma <- robreg$s
             est <- cbind(V1=B[1,] + as.matrix(regbasis[misspat==npat,-1])%*%B[-1,],regbasis[misspat==npat,-1])
-            est[,1] <- est[,1] - sigma*(dnorm((phi[,1]-est[,1])/sigma)/pnorm((phi[,1]-est[,1])/sigma))
+            if (imp.missing==FALSE){
+              est[,1] <- est[,1] - sigma*(dnorm((phi[,1]-est[,1])/sigma)/pnorm((phi[,1]-est[,1])/sigma))
+            }
             est <- t(apply(est,1,inv.ilr))
             est <- est[,1]*(feeder[misspat==npat,1]/est[,2])
             X[misspat==npat,p] <- est
@@ -307,10 +330,12 @@ lrEM <- function(X,label=NULL,dl=NULL,rob=FALSE,ini.cov=c("complete.obs","multRe
   
   if (suppress.print==FALSE){
     if (alt.in) {
-      cat("Warning: samples with only one observed component were found \n")
-      for (i in 2:length(alt.pat)){
-        cat(paste("  Pattern no.",alt.pat[i],"was imputed using multiplicative simple replacement \n"))
-        cat("   Affected samples id: "); cat(alt.mr[[i]]); cat("\n\n")
+      if (imp.missing==FALSE){
+        warning("Censoring patterns with only one observed component in the data set.")
+        cat("Censored samples with only one observed component imputed by simple multiplicative replacement. \n")
+        for (i in 2:length(alt.pat)){
+          cat("Row numbers: "); cat(alt.mr[[i]]); cat("\n\n")
+        }
       }
     }
   cat(paste("No. iterations to converge: ",niters,"\n\n"))
